@@ -460,6 +460,13 @@ static void Band_DFT_MO_Col(
       /* diagonalization of C */
       Eigen_PHH(mpi_comm_level1,C[spin],ko[spin],n,n,0);
 
+      if (3<=level_stdout){
+	printf("  kloop %i, k1 k2 k3 %10.6f %10.6f %10.6f\n",kloop,k1,k2,k3);
+	for (i1=1; i1<=n; i1++){
+	  printf("  Eigenvalues of Hks  %2d  %15.12f\n",i1,ko[spin][i1]);
+	}
+      }
+
       for (i1=1; i1<=n; i1++){
         EIGEN[kloop][spin][i1] = ko[spin][i1];
       }
@@ -524,6 +531,7 @@ static void Band_DFT_MO_Col(
 	if (x_cut<=x)  x = x_cut;
 	if (SpinP_switch==0) FermiF = 2.0/(1.0 + exp(x));
 	else                 FermiF = 1.0/(1.0 + exp(x));
+
 	if      (SpinP_switch==0 && 1.0<FermiF) Bulk_HOMO[kloop][spin] = i1;
 	else if (SpinP_switch==1 && 0.5<FermiF) Bulk_HOMO[kloop][spin] = i1;
       }      
@@ -1489,8 +1497,8 @@ static void Band_DFT_MO_NonCol(
     nhomos = num_HOMOs;
     nlumos = num_LUMOs;
 
-    if ( (Bulk_HOMO[kloop][0]-nhomos+1)<1 ) nhomos = Bulk_HOMO[kloop][0];
-    if ( (Bulk_HOMO[kloop][0]+nlumos)>n1 )  nlumos = n1 - Bulk_HOMO[kloop][0];
+    if ( (Bulk_HOMO[kloop][0]-nhomos+1)<1 )  nhomos = Bulk_HOMO[kloop][0];
+    if ( (Bulk_HOMO[kloop][0]+nlumos)>MaxN)  nlumos = MaxN - Bulk_HOMO[kloop][0];
 
     /* HOMOs */
 
