@@ -18,6 +18,7 @@
 #include "lapack_prototypes.h"
 #include "mpi.h"
 #include <omp.h>
+#include "f77func.h"
 
 
  
@@ -51,7 +52,13 @@ void Eigen_PReHH(MPI_Comm MPI_Current_Comm_WD,
                  double **ac, double *ko, int n, int EVmax, int bcast_flag)
 {
 
-  if      (scf_eigen_lib_flag==0 || n<100)
+  if (n<20)
+    Eigen_lapack(ac,ko,n,EVmax);
+
+  else if (rediagonalize_flag_overlap_matrix_ELPA1==1)
+    Eigen_ELPA1_Re(MPI_Current_Comm_WD, ac, ko, n, EVmax, bcast_flag); 
+
+  else if (scf_eigen_lib_flag==0 || n<100)
     Eigen_Improved_PReHH(MPI_Current_Comm_WD, ac, ko, n, EVmax, bcast_flag); 
 
   else if (scf_eigen_lib_flag==1)
@@ -75,7 +82,10 @@ void Eigen_ELPA1_Re(MPI_Comm MPI_Current_Comm_WD,
 
   int na = n;
   int nev = EVmax;
+  int nblk = 4;
+  /*
   int nblk = 16;
+  */
 
   int np_rows, np_cols, na_rows, na_cols;
   int myid, numprocs, my_prow, my_pcol;
@@ -168,7 +178,7 @@ void Eigen_ELPA1_Re(MPI_Comm MPI_Current_Comm_WD,
   mpi_comm_rows_int = MPI_Comm_c2f(mpi_comm_rows);
   mpi_comm_cols_int = MPI_Comm_c2f(mpi_comm_cols);
 
-  solve_evp_real_(&na, &nev, a, &na_rows, &ko[1], z, &na_rows, &nblk, &mpi_comm_rows_int, &mpi_comm_cols_int);
+  F77_NAME(solve_evp_real,SOLVE_EVP_REAL)(&na, &nev, a, &na_rows, &ko[1], z, &na_rows, &nblk, &mpi_comm_rows_int, &mpi_comm_cols_int);
 
   MPI_Comm_free(&mpi_comm_rows);
   MPI_Comm_free(&mpi_comm_cols);
@@ -275,7 +285,7 @@ int numrocC(int N, int NB, int IPROC, int ISRCPROC, int NPROCS)
 void Eigen_Improved_PReHH(MPI_Comm MPI_Current_Comm_WD, 
                           double **ac, double *ko, int n, int EVmax, int bcast_flag)
 {
-  double ABSTOL=1.0e-13;
+  double ABSTOL=LAPACK_ABSTOL;
   double *q,*p;
   double **ad,*b1,*uu,
     s1,s2,s3,ss,u1,u2,r,p1,my_r,my_s2, 
@@ -322,12 +332,11 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Current_Comm_WD,
 
   if ( numprocs<=EVmax ){
 
-    av_num = (double)EVmax/(double)numprocs;
-
-    for (ID=0; ID<numprocs; ID++){
-      is1[ID] = (int)(av_num*(double)ID) + 1; 
-      ie1[ID] = (int)(av_num*(double)(ID+1)); 
-    }
+    av_num = (double)EVmax/(double)numprocs;                                                                
+    for (ID=0; ID<numprocs; ID++){                                                                          
+      is1[ID] = (int)(av_num*(double)ID) + 1;                                                               
+      ie1[ID] = (int)(av_num*(double)(ID+1));                                                               
+    }                                               
 
     is1[0] = 1;
     ie1[numprocs-1] = EVmax; 
@@ -503,6 +512,7 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Current_Comm_WD,
       sum = sum + ac[j][i] * ac[j][i];
     }
     sum = 1.0/sqrt(sum);
+
     for (i=1; i<=n; i++){
       ac[j][i] = ac[j][i] * sum;
     }
@@ -635,7 +645,7 @@ void myHH( MPI_Comm MPI_Current_Comm_WD, int numprocs, int myid, int n, double *
            MPI_Request *request_recv, 
            MPI_Status *stat_send)
 {
-  double ABSTOL=1.0e-13; 
+  double ABSTOL=LAPACK_ABSTOL; 
   int jj1,i,j,i1,j1,i1s,is0,ID1,ID0,k1,i2,n2;
   int nump,num0,k0,num,count,ID;
   int IDS,IDR,num1,tag=999;
@@ -680,7 +690,6 @@ void myHH( MPI_Comm MPI_Current_Comm_WD, int numprocs, int myid, int n, double *
   if ( numprocs<=(n-1) ){
 
     av_num = (double)(n-1)/(double)numprocs;
-
     for (ID=0; ID<numprocs; ID++){
       is2[ID] = (int)(av_num*(double)ID) + 2;
       ie2[ID] = (int)(av_num*(double)(ID+1)) + 1;
@@ -1024,7 +1033,7 @@ void myHH( MPI_Comm MPI_Current_Comm_WD, int numprocs, int myid, int n, double *
 void Eigen_Original_PReHH(MPI_Comm MPI_Current_Comm_WD, 
                   double **ac, double *ko, int n, int EVmax, int bcast_flag)
 {
-  double ABSTOL=1.0e-13;
+  double ABSTOL=LAPACK_ABSTOL;
   double **ad,*D,*E,
     *b1,*u,*uu,
     *p,*q,
@@ -1090,12 +1099,6 @@ void Eigen_Original_PReHH(MPI_Comm MPI_Current_Comm_WD,
     }
   }
 
-  /*
-  for (ID=0; ID<numprocs; ID++){
-    printf("myid=%2d ID=%2d is1=%3d ie1=%3d\n",myid,ID,is1[ID],ie1[ID]);
-  }
-  */
-
   /****************************************************
     allocation of arrays:
   ****************************************************/
@@ -1190,8 +1193,6 @@ void Eigen_Original_PReHH(MPI_Comm MPI_Current_Comm_WD,
 
       if (measure_time==1) dtime(&Stime1);
 
-      printf("i=%2d r=%18.15f\n",i,r);
-
       r = r / u2;
       for (i1=i+1; i1<=n; i1++){
 	q[i1] = p[i1] - r * u[i1];
