@@ -27,20 +27,81 @@ static void Eigen_HH(double **ac, double *ko, int n, int EVmax);
    
 void Eigen_lapack(double **a, double *ko, int n, int EVmax)
 {
+  static int solver_flag=0;
+  int i,j,k,po,okay_flag,iterN;
+  double sum;
+  double **b;
 
-  Eigen_HH(a, ko, n, EVmax); 
+  b=(double**)malloc(sizeof(double*)*(n+1));
+  for (i=0; i<(n+1); i++){
+    b[i]=(double*)malloc(sizeof(double)*(n+1));
+  }
 
-  /*
-  Eigen_lapack_x(a, ko, n, EVmax); 
-  */
+  for (i=0; i<(n+1); i++){
+    for (j=0; j<(n+1); j++){
+      b[i][j] = a[i][j]; 
+    }
+  }
 
-  /*
-  Eigen_HHQR(a, ko, n, EVmax); 
-  */
+  iterN = 1;
+  okay_flag = 0;
+
+  do {
+
+    for (i=0; i<(n+1); i++){
+      for (j=0; j<(n+1); j++){
+	a[i][j] = b[i][j]; 
+      }
+    }
+
+    if      (solver_flag==0) Eigen_lapack_x(a, ko, n, EVmax);
+    else if (solver_flag==1) Eigen_HH(a, ko, n, EVmax);  
+    else if (solver_flag==2) Eigen_lapack_d(a, ko, n, EVmax); 
+
+    po = 0; 
+
+    i = 1;
+    do {
+      j = 1;
+      do {
+
+	sum = 0.0;
+	for (k=1; k<=n; k++){
+	  sum += a[i][k]*a[j][k]; 
+	}
+
+	if      (i==j && 0.00001<fabs(sum-1.0)){ po = 1; }
+	else if (i!=j && 0.00001<fabs(sum))    { po = 1; }
+
+        j = j + 20;
+
+      } while (po==0 && j<=EVmax); 
+
+      i = i + 20;
+
+    } while (po==0 && i<=EVmax); 
+
+    if (po==1){
+      solver_flag++; 
+      solver_flag = solver_flag % 3;
+    }
+    else {
+      okay_flag = 1;
+    }
+
+    /*
+    printf("iterN=%2d solver_flag=%2d po=%2d okay_flag=%2d\n",iterN,solver_flag,po,okay_flag);
+    */
+
+    iterN++;
+
+  } while (okay_flag==0 && iterN<4);  
+
+  for (i=0; i<(n+1); i++){
+    free(b[i]);
+  }
+  free(b);
 
-  /*
-  Eigen_lapack_d(a, ko, n, EVmax);
-  */
 } 
 
 
@@ -57,7 +118,7 @@ void Eigen_lapack_x(double **a, double *ko, int n0, int EVmax)
     output: ko[n];    eigenvalues 
   */
     
-  char *name="Eigen_lapack";
+  char *name="Eigen_lapack_x";
 
   char  *JOBZ="V";
   char  *RANGE="I";
@@ -67,7 +128,7 @@ void Eigen_lapack_x(double **a, double *ko, int n0, int EVmax)
   INTEGER LDA=n0;
   double VL,VU; /* dummy */
   INTEGER IL,IU; 
-  double ABSTOL=1.0e-13;
+  double ABSTOL=LAPACK_ABSTOL;
   INTEGER M;
 
   double *A,*Z;
@@ -125,9 +186,12 @@ void Eigen_lapack_x(double **a, double *ko, int n0, int EVmax)
     ko[i]= ko[i-1];
   }
 
+  /*
   if (INFO>0) {
     printf("\n%s: error in dsyevx_, info=%d\n\n",name,INFO);
   }
+  */
+
   if (INFO<0) {
      printf("%s: info=%d\n",name,INFO);
      exit(10);
@@ -150,7 +214,7 @@ void Eigen_lapack_d(double **a, double *ko, int n0, int EVmax)
     output: ko[n];    eigenvalues 
   */
     
-  static char *name="Eigen_lapack";
+  static char *name="Eigen_lapack_d";
 
   char  *JOBZ="V";
   char  *UPLO="L";
@@ -159,7 +223,7 @@ void Eigen_lapack_d(double **a, double *ko, int n0, int EVmax)
   INTEGER LDA=n;
   double VL,VU; /* dummy */
   INTEGER IL,IU; 
-  double ABSTOL=1.0e-13;
+  double ABSTOL=LAPACK_ABSTOL;
   INTEGER M;
 
   double *A;
@@ -215,9 +279,12 @@ void Eigen_lapack_d(double **a, double *ko, int n0, int EVmax)
     ko[i]= ko[i-1];
   }
 
+  /*
   if (INFO>0) {
      printf("\n%s: error in dsyevd_, info=%d\n\n",name,INFO);
   }
+  */
+
   if (INFO<0) {
      printf("%s: info=%d\n",name,INFO);
      exit(10);
@@ -239,7 +306,7 @@ void Eigen_lapack_r(double **a, double *ko, int n0, int EVmax)
     output: ko[n];    eigenvalues 
   */
     
-  static char *name="Eigen_lapack";
+  static char *name="Eigen_lapack_r";
 
   char  *JOBZ="V";
   char  *RANGE="I";
@@ -249,7 +316,7 @@ void Eigen_lapack_r(double **a, double *ko, int n0, int EVmax)
   INTEGER LDA=n;
   double VL,VU; /* dummy */
   INTEGER IL,IU; 
-  double ABSTOL=1.0e-13;
+  double ABSTOL=LAPACK_ABSTOL;
   INTEGER M;
 
   double *A,*Z;
@@ -310,9 +377,12 @@ void Eigen_lapack_r(double **a, double *ko, int n0, int EVmax)
     ko[i]= ko[i-1];
   }
 
+  /*
   if (INFO>0) {
      printf("\n%s: error in dsyevr_, info=%d\n\n",name,INFO);
   }
+  */
+
   if (INFO<0) {
      printf("%s: info=%d\n",name,INFO);
      exit(10);
@@ -340,7 +410,7 @@ void Eigen_HH(double **ac, double *ko, int n, int EVmax)
 
   ***********************************************************************/
 
-  double ABSTOL=1.0e-13;
+  double ABSTOL=LAPACK_ABSTOL;
   double **ad,*D,*E,
                 *b1,*u,*uu,
                 *p,*q,*s,*c,
@@ -794,7 +864,7 @@ void Eigen_HHQR(double **ac, double *ko, int n, int EVmax)
 
   ***********************************************************************/
 
-  double ABSTOL=1.0e-13;
+  double ABSTOL=LAPACK_ABSTOL;
 
   double **ad,**aq,**b,
                 *b1,*u,*uu,
