@@ -2,12 +2,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include <mpi.h>
-#endif
 
 #include "tran_prototypes.h"
 #include "lapack_prototypes.h"
@@ -61,7 +56,7 @@ int LU_Zinverse_Kino(int n, dcomplex *A)
     int i,j;
     dcomplex *B,*C;
     dcomplex alpha,beta;
-    double eps=1.0e-10;
+    double eps=1.0e-13;
     alpha.r=1.0; alpha.i=0.0;
     beta.r=0.0; beta.i=0.0;
     B=(dcomplex*)malloc(sizeof(dcomplex)*n*n);
