@@ -15,18 +15,13 @@
 #include <math.h>
 #include <time.h>
 #include "openmx_common.h"
+#include "mpi.h"
 
 #ifdef c_complex
 #include <complex.h>
 #endif
 
 
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
-#include "mpi.h"
-#endif
-
 
 void OM_onsite( char *mode );
 void OM_dual( char *mode );
