@@ -90,7 +90,7 @@ void Show_DFT_DATA(char *argv[])
     /* count the number of dat files */
 
     if(( dp = opendir(dir) ) == NULL ){
-      printf("could not find the directry '%s'\n",input_dir);
+      printf("could not find the directory '%s'\n",input_dir);
       MPI_Finalize();
       exit(0);
     }
@@ -110,7 +110,7 @@ void Show_DFT_DATA(char *argv[])
     /* store the name of dat files */
 
     if(( dp = opendir(dir) ) == NULL ){
-      printf("could not find the directry '%s'\n",input_dir);
+      printf("could not find the directory '%s'\n",input_dir);
       MPI_Finalize();
       exit(0);
     }
