@@ -19,13 +19,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include <mpi.h>
-#endif
-
 #include "tran_prototypes.h"
 #include "tran_variables.h"
 
