@@ -11,13 +11,7 @@
 
 #include <stdio.h>
 #include <string.h>
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include <mpi.h>
-#endif
-
 #include "tran_prototypes.h"
 
 void TRAN_Credit(MPI_Comm comm1)
