@@ -14,17 +14,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#include <mpi.h>
+#include "tran_prototypes.h"
+#include "tran_variables.h"
 
 #define MEASURE_TIME 0
 
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
-#include <mpi.h>
-#endif
 
-#include "tran_prototypes.h"
-#include "tran_variables.h"
 
 void dtime(double *);
 
@@ -204,13 +200,13 @@ void TRAN_Set_CentOverlap(
 	tnoA = Spe_Total_CNO[wanA];
 	Anum = MP[GA_AN];
 
-	GA_AN_e = TRAN_Original_Id[GA_AN];
-	Anum_e = MP_e[iside][GA_AN_e];
-
 	for (GB_AN=1; GB_AN<=atomnum; GB_AN++){
 
 	  if ( TRAN_region[GA_AN]==12 && TRAN_region[GB_AN]==12 ) {
 
+	    GA_AN_e = TRAN_Original_Id[GA_AN];
+  	    Anum_e = MP_e[iside][GA_AN_e];
+
 	    wanB = WhatSpecies[GB_AN];
 	    tnoB = Spe_Total_CNO[wanB];
 	    Bnum = MP[GB_AN];
@@ -256,13 +252,13 @@ void TRAN_Set_CentOverlap(
 	tnoA = Spe_Total_CNO[wanA];
 	Anum = MP[GA_AN];
 
-	GA_AN_e = TRAN_Original_Id[GA_AN];
-	Anum_e = MP_e[iside][GA_AN_e]; /* = Anum */
-
 	for (GB_AN=1; GB_AN<=atomnum; GB_AN++){
 
 	  if ( TRAN_region[GA_AN]==13 && TRAN_region[GB_AN]==13 )  {
 
+	    GA_AN_e = TRAN_Original_Id[GA_AN];
+	    Anum_e = MP_e[iside][GA_AN_e]; /* = Anum */
+
 	    wanB = WhatSpecies[GB_AN];
 	    tnoB = Spe_Total_CNO[wanB];
 	    Bnum = MP[GB_AN];
