@@ -1,3 +1,14 @@
+/**********************************************************************
+  TRAN_Set_MP.c:
+
+  TRAN_Set_MP.c is a subroutine to set an array 'MP'.
+
+  Log of TRAN_Set_MP.c:
+
+     11/Dec/2005   Released by H.Kino
+
+***********************************************************************/
+
 #include <stdio.h>
 #include <math.h>
 #include <string.h>
@@ -14,7 +25,7 @@ void TRAN_Set_MP(
  /* setup MP */
   Anum = 1;
   for (i=1; i<=atomnum; i++){
-    if (job) MP[i]=Anum; 
+    if (job) MP[i] = Anum; 
     wanA = WhatSpecies[i];
     tnoA = Spe_Total_CNO[wanA];
     Anum = Anum + tnoA;
