@@ -10,12 +10,7 @@
 ***********************************************************************/
 
 #include <stdio.h>
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include <mpi.h>
-#endif
-
 #include "tran_prototypes.h"
 
 /*
