@@ -15,13 +15,15 @@
 
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <math.h>
 #include <string.h>
 
 #define MAXBUF 1024
 #define fp_bsize         1048576     /* buffer size for setvbuf */
 
-/* #define DEBUG */
+/* #define DEBUG  */
+
 #ifdef DEBUG
 #undef DEBUG
 #endif
@@ -204,6 +206,7 @@ main(int argc, char **argv)
   for (j=0;j<(nkpath+1);j++) {
     kline[j]=(double*)malloc(sizeof(double)*(m_perpath+1));
   }
+
 #ifdef DEBUG
   printf("<kpath end>\n");
 #endif
@@ -349,7 +352,7 @@ main(int argc, char **argv)
     for(l=1;l<=meig;l++) {
       for (ik=1;ik<=nkpath;ik++) {
 	for (i_perpath=1;i_perpath<=n_perpath[ik]; i_perpath++) {
-	  fprintf(fp, "%lf %lf\n",kline[ik][i_perpath],
+	  fprintf(fp, "%lf %15.12f\n",kline[ik][i_perpath],
 		  (EIGEN[spin][ik][i_perpath][l]-ChemP)*Unit );
 	}
 	fprintf(fp, "\n\n");
@@ -379,7 +382,7 @@ main(int argc, char **argv)
   }
   printf("%s is made\n",fname);
 
-  fprintf(fp,"set data s l\n");
+  fprintf(fp,"set style data lines\n");
   fprintf(fp,"set nokey\n");
   fprintf(fp,"set zeroaxis\n");
   fprintf(fp,"set ytics 1\n");
