@@ -43,6 +43,7 @@ int atomnum1,atomnum2;
 double **Gxyz1;
 double **Gxyz2;
 double BL0;
+char **Species;
 
 int main(int argc, char *argv[]) 
 {
@@ -397,7 +398,7 @@ double minimize_rmsd()
                     + Rot[3][3]*(Gxyz2[ct_AN][3]-Fz);
   }
 
-  printf(" generate a vector file, dgeo_vec.txt\n\n");
+  printf(" generate a vector file, dgeo_vec.xsf\n\n");
   generate_vec(Gxyz1, xyz2);
 
   /* freeing of array */  
@@ -453,6 +454,11 @@ void read_xyzfiles(char file1[100], char file2[100])
       Gxyz1[ct_AN] = (double*)malloc(sizeof(double)*4);
     }
 
+    Species = (char**)malloc(sizeof(char*)*(atomnum1+1));
+    for (ct_AN=0; ct_AN<=atomnum1; ct_AN++){
+      Species[ct_AN] = (char*)malloc(sizeof(char)*15);
+    }
+
     for (ct_AN=1; ct_AN<=atomnum1; ct_AN++){
       fscanf(fp,"%s %lf %lf %lf %lf %lf %lf",
              stmp,
@@ -482,7 +488,7 @@ void read_xyzfiles(char file1[100], char file2[100])
 
     for (ct_AN=1; ct_AN<=atomnum2; ct_AN++){
       fscanf(fp,"%s %lf %lf %lf %lf %lf %lf",
-             stmp,
+             Species[ct_AN],
              &Gxyz2[ct_AN][1],&Gxyz2[ct_AN][2],&Gxyz2[ct_AN][3],
              &tmp1,&tmp2,&tmp3);
     }
@@ -741,7 +747,7 @@ double minimize_md()
                     + Rot[3][3]*(Gxyz2[ct_AN][3]-Fz);
   }
 
-  printf(" generate a vector file, dgeo_vec.txt\n\n");
+  printf(" generate a vector file, dgeo_vec.xsf\n\n");
   generate_vec(Gxyz1, xyz2);
 
   /* freeing of array */  
@@ -817,7 +823,7 @@ void generate_vec(double **xyz1, double **xyz2)
   FILE *fp;
   char fp_buf[fp_bsize];          /* setvbuf */
 
-  sprintf(fname,"dgeo_vec.txt");
+  sprintf(fname,"dgeo_vec.xsf");
 
   if ((fp = fopen(fname,"w")) != NULL){
 
@@ -825,42 +831,20 @@ void generate_vec(double **xyz1, double **xyz2)
     setvbuf(fp,fp_buf,_IOFBF,fp_bsize);  /* setvbuf */
 #endif
 
-    xmin = 10.0e+10;
-    ymin = 10.0e+10;
-    zmin = 10.0e+10;
-    xmax = -10.0e+10;
-    ymax = -10.0e+10;
-    zmax = -10.0e+10;
-
-    for (AN1=1; AN1<=atomnum1; AN1++){
-      if (xyz1[AN1][1]<xmin)  xmin = xyz1[AN1][1];
-      if (xyz1[AN1][2]<ymin)  ymin = xyz1[AN1][2];
-      if (xyz1[AN1][3]<zmin)  zmin = xyz1[AN1][3];
-      if (xmax<xyz1[AN1][1])  xmax = xyz1[AN1][1];
-      if (ymax<xyz1[AN1][2])  ymax = xyz1[AN1][2];
-      if (zmax<xyz1[AN1][3])  zmax = xyz1[AN1][3];
-    }
-
-    fprintf(fp,"3 200\n");
-    fprintf(fp,"%4d %4d %4d\n",atomnum1,0,0);
-    fprintf(fp,"%13.3E %13.3E\n",xmin,xmax);
-    fprintf(fp,"%13.3E %13.3E\n",ymin,ymax);
-    fprintf(fp,"%13.3E %13.3E\n",zmin,zmax);
-
+    fprintf(fp,"ATOMS\n");
     for (AN1=1; AN1<=atomnum1; AN1++){
       dx = xyz2[AN1][1] - xyz1[AN1][1];    
       dy = xyz2[AN1][2] - xyz1[AN1][2];    
       dz = xyz2[AN1][3] - xyz1[AN1][3];    
-      fprintf(fp,"%13.3E %13.3E %13.3E %13.3E %13.3E %13.3E\n",
-	      xyz1[AN1][1],xyz1[AN1][2],xyz1[AN1][3],
+      fprintf(fp,"%3s  %13.3E %13.3E %13.3E %13.3E %13.3E %13.3E\n",
+	      Species[AN1],xyz1[AN1][1],xyz1[AN1][2],xyz1[AN1][3],
 	      dx,dy,dz);
-
     }  
     fclose(fp);
 
   }
   else{
-    printf("Failure of saving the vector file\n");
+    printf("Failure of saving the xsf file.\n");
   }
   
 } 
